% Compute point estimator for input data y given a loss function
% for use with GIRP partitioning (i.e. cut) problem
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Inputs:  
% y:           n x 1 input vector to isotonic regression, e.g. observations
% options:     same options as to GIRP(.) function
% inds:        indices of observations in the group to be cut (for use when there are weights)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Outputs:  
% yfit:        point estimator for given loss function
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Modified by Ronny Luss, May 2011
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [yfit] = GIRP_estimator(y,options,inds)

switch options.est_type
    case 'L2'
        yfit=mean(y);
    case 'pnorm'
        yfit=pnorm_estimator(y,options.p);
    case 'L2_weighted'
        yfit=(options.weights(inds)'*y)/sum(options.weights(inds));
    case 'huberloss'
        yfit=huberloss_estimator(y,options.delta);
    case 'huberloss_weighted'
        list=groupMerge(options.group(inds));
        yfit=huberloss_estimator(options.y(list),options.delta);
    case 'poisson'
        yfit=mean(y);
    case 'poisson_weighted'
        list=groupMerge(options.group(inds));
        yfit=mean(options.y(list));
end