% Compute the negative of the derivative of a function evaluated at the point estimator
% for use with GIRP partitioning (i.e. cut) problem
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Inputs:  
% y:           n x 1 input vector to isotonic regression, e.g. observations
% estimator:   point estimator for isotonic regresson over entire group
% options:     same options as to GIRP(.) function
% inds:        indices of observations in the group to be cut (for use when there are weights)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Outputs:  
% v:           n x 1 output vector of values to be used in objective of partitioning problem 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Modified by Ronny Luss, May 2011
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [v] = cut_value(y,estimator,options,inds)

switch options.est_type
    case 'L2'
        v=2*(y-estimator);
    case 'pnorm'
        v=options.p*(y-estimator).*abs(y-estimator).^(options.p-2);
    case 'L2_weighted'
        v=2*(y-estimator).*options.weights(inds);
    case 'huberloss'
        ind1=find(abs(estimator-y)<=options.delta);
        ind2=find(abs(estimator-y)>options.delta);
        v=zeros(length(y),1);
        v(ind1)=y(ind1)-estimator;
        v(ind2)=sign(y(ind2)-estimator)*options.delta;
    case 'huberloss_weighted'
        ind1=find(abs(estimator-options.y)<=options.delta);
        ind2=find(abs(estimator-options.y)>options.delta);
        w=zeros(length(options.y),1);
        w(ind1)=options.y(ind1)-estimator;
        w(ind2)=sign(options.y(ind2)-estimator)*options.delta;
        v=zeros(length(y),1);
        for i=1:length(y)
           v(i)=sum(w(options.group{inds(i)})); 
        end
    case 'poisson'
        v=-ones(length(y),1)+y/estimator;
    case 'poisson_weighted'
        w=-ones(length(options.y),1)+options.y/estimator;
        v=zeros(length(y),1);
        for i=1:length(y)
           v(i)=sum(w(options.group{inds(i)})); 
        end
end



