%  Computes the delta-huber loss estimator on data y using binary search on derivatives 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Inputs:  
% y:           n x 1 input vector to isotonic regression, e.g. observations
% delta:       delta parameter for (weighted) huber isotonic regression
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Outputs:  
% x:           point estimator for huber loss function
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Modified by Ronny Luss, May 2011
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [x] = huberloss_estimator(y,delta)

% BINARY SEARCH!!!
a=min(y);ind1=find(abs(a-y)<=delta);ind2=find(abs(a-y)>delta);f_a=sum(a-y(ind1))+delta*sum(sign(a-y(ind2)));
c=max(y);ind1=find(abs(c-y)<=delta);ind2=find(abs(c-y)>delta);f_c=sum(c-y(ind1))+delta*sum(sign(c-y(ind2)));
x=(a+c)/2;ind1=find(abs(x-y)<=delta);ind2=find(abs(x-y)>delta);f_x=sum(x-y(ind1))+delta*sum(sign(x-y(ind2)));
while abs(a-c)>.000000001
    if sign(f_a)==sign(f_x)
       a=x;ind1=find(abs(a-y)<=delta);ind2=find(abs(a-y)>delta);f_a=sum(a-y(ind1))+delta*sum(sign(a-y(ind2)));
    else
       c=x;ind1=find(abs(c-y)<=delta);ind2=find(abs(c-y)>delta);f_c=sum(c-y(ind1))+delta*sum(sign(c-y(ind2)));
    end    
    x=(a+c)/2;ind1=find(abs(x-y)<=delta);ind2=find(abs(x-y)>delta);f_x=sum(x-y(ind1))+delta*sum(sign(x-y(ind2)));
end
