% Compute loss function for training and testing data
% for use with GIRP partitioning (i.e. cut) problem
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Inputs:  
% yTrain:      n1 x 1 vector of training observations
% yTest:       n2 x 1 vector of testing observations
% pathGIRP:    n1 x count matrix where columns are the isotonic solutions
%                 along the GIRP path, and count is the length of the path
% yPred:       predictions of testing points along all isotonic models
%                 given by input pathGIRP
% options:      same options as to GIRP(.) function.  here used for options.lossFunction
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Outputs:  
% errorTest:    error on testing data 
% errorTrain:   error on training data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Modified by Ronny Luss, May 2011
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [errorTest,errorTrain] = predictLossFunction(yTrain,yTest,pathGIRP,yPred,options)

switch options.lossFunction
    case 'mse'
        errorTest=mean((yPred-yTest*ones(1,size(yPred,2))).^2,1);
        errorTrain=mean((pathGIRP-yTrain*ones(1,size(yPred,2))).^2,1);
    case 'poisson' % negative poisson loglikelihood        
        % DEBUG: must bound predictions away from zero        
        % Bound the predictions for  testing (yPred) away from zero
        for j=1:size(yPred,2)
            if min(yPred(:,j))==0
                indzeros=find(yPred(:,j)==0);
                yPred(indzeros,j)=.001;
            end
        end
        % Bound the predictions for training (pathGIRP) away from zero
        for j=1:size(pathGIRP,2)
            if min(pathGIRP(:,j))==0
                indzeros=find(pathGIRP(:,j)==0);
                pathGIRP(indzeros,j)=.001;
            end
        end
        errorTest=-sum((yTest*ones(1,size(yPred,2))).*log(yPred)-yPred-logbase_e_factorial(yTest)*ones(1,size(yPred,2)),1);
        errorTrain=-sum((yTrain*ones(1,size(yPred,2))).*log(pathGIRP)-pathGIRP-logbase_e_factorial(yTrain)*ones(1,size(yPred,2)),1);
end