% Test GIRP on L2 Isotonic Regression with ternary data and produce GIRP path

% Modify paths
addpath('C:/Program Files/Mosek/6/toolbox/r2009b');
addpath('..')
addpath('../subRoutines')

n1=12000; % number of training observations
n2=3000; % number of testing observations
n=n1+n2;
d=4;   % dimension
sigma=d;

rand('state',25);
randn('state',20);
% observation matrix
data=floor(rand(n,d)*3);
% take y_i=sum_d((x_id)^2)
y=prod(data,2);
y=y+sigma*randn(n,1); % add noise N(0,sigma^2)
dataTrain=data(1:n1,:);yTrain=y(1:n1);
dataTest=data(n1+1:n1+n2,:);yTest=y(n1+1:n1+n2);

[newdata,newy,weights]=WeightedUniqueDataL2(dataTrain,yTrain);

% Set options for weighted GIRP
options=[];
options.est_type='L2_weighted';
options.lossFunction='mse';
options.maxNumPartitions=500;

disp('Start creating adjacency matrix for creating training data model.');tic;    
[A,ind_A,ind_original]=adjacency_matrix(newdata);
disp(['Done creating adjacency matrix after ',num2str(toc),' seconds.']);
newy=newy(ind_A);
newdata=newdata(ind_A,:);
options.weights=weights(ind_A);

disp('Starting GIRP.');tic;    
[x1,cutInds,iter,violation_flag]=GIRP(newy,A,options);
disp(['Done running GIRP after ',num2str(toc),' seconds.']);

% NOTE: a faster predict wrapper could be written using the uniqueness of points in dataTest
[errorTest,errorTrain]=predictWrapper(newy,yTest,newdata,dataTest,A,cutInds,1000,options);

clf;
figure(1);
semilogx(1:size(cutInds,1),errorTrain,'-ko','LineWidth',2);
hold on;
title('Error on Training Data');
xlabel('GIRP Path');
ylabel('MSE');
axis([0,500,min(errorTrain),max(errorTrain)]);
hold off;

figure(2);
semilogx(1:size(cutInds,1),errorTest,'-ko','LineWidth',2);
hold on;
[minMSE,minPath]=min(errorTest);
semilogx(minPath,errorTest(minPath),'-rd','MarkerSize',10,'MarkerFaceColor','r');
title('Error on Testing Data');
xlabel('GIRP Path');
ylabel('MSE');
axis([0,500,min(errorTest),max(errorTest)]);
hold off;
