% Test GIRP on Poisson Isotonic Regression and produce GIRP path

% Modify paths
addpath('C:/Program Files/Mosek/6/toolbox/r2009b');
addpath('..')
addpath('../subRoutines')

n1=12000; % number of training observations
n2=3000; % number of testing observations
n=n1+n2;
d=6;   % dimension

rand('state',30);
% observation matrix
data=rand(n,d)*10;
% take y_i=Poisson(prod_d(sqrt(x_id)))
y=ones(n,1);
for i=1:d
    y=y.*sqrt(data(:,i));
end    
y=poissrnd(y);
dataTrain=data(1:n1,:);yTrain=y(1:n1);
dataTest=data(n1+1:n1+n2,:);yTest=y(n1+1:n1+n2);

% Set options for GIRP
options=[];
options.est_type='poisson';
options.lossFunction='poisson';
options.maxNumPartitions=500;

disp('Start creating adjacency matrix for creating training data model.');tic;    
[A,ind_A,ind_original]=adjacency_matrix(dataTrain);
disp(['Done creating adjacency matrix after ',num2str(toc),' seconds.']);
yTrain=yTrain(ind_A);
dataTrain=dataTrain(ind_A,:);

disp('Starting GIRP.');tic;    
[x1,cutInds,iter,violation_flag]=GIRP(yTrain,A,options);
disp(['Done running GIRP after ',num2str(toc),' seconds.']);

[errorTest,errorTrain]=predictWrapper(yTrain,yTest,dataTrain,dataTest,A,cutInds,1000,options);

clf;
figure(1);
semilogx(1:size(cutInds,1),errorTrain,'-ko','LineWidth',2);
hold on;
title('Error on Training Data');
xlabel('GIRP Path');
ylabel('Poisson Neg. Loglikelihood');
axis([0,500,min(errorTrain),max(errorTrain)]);
hold off;

figure(2);
semilogx(1:size(cutInds,1),errorTest,'-ko','LineWidth',2);
hold on;
[minMSE,minPath]=min(errorTest);
semilogx(minPath,errorTest(minPath),'-rd','MarkerSize',10,'MarkerFaceColor','r');
title('Error on Testing Data');
xlabel('GIRP Path');
ylabel('Poisson Neg. Loglikelihood');
axis([0,500,min(errorTest),max(errorTest)]);
hold off;
